package com.gic.enterprise.service;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.dto.union.UnionEnterpriseAuthDTO;
import com.gic.enterprise.dto.union.UnionEnterpriseAuthListDTO;
import com.gic.enterprise.dto.union.UnionEnterpriseAuthResDTO;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * 联合商户
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/2/3 1:47 PM 
 */
public interface UnionEnterpriseAuthApiService {

    /**
     * 创建联合商户
     * @param ownEnterpriseId
     * @param unionEnterpriseId
     * @return
     */
    ServiceResponse<Void> saveUnionEnterprise(@NotNull Integer ownEnterpriseId, @NotNull Integer unionEnterpriseId);

    /**
     * 授权资源
     * @param dto
     * @return
     */
    ServiceResponse<Void> authResource(@Valid UnionEnterpriseAuthResDTO dto);

    /**
     * 查询联合商户记录
     * @param ownEnterpriseId
     * @param unionEnterpriseId
     * @return
     */
    ServiceResponse<UnionEnterpriseAuthDTO> getUnionEnterpriseAuth(@NotNull Integer ownEnterpriseId, @NotNull Integer unionEnterpriseId);

    /**
     * 资源重新授权
     * @param ownEnterpriseId
     * @param unionEnterpriseId
     * @param resourceType UnionEnterpriseAuthResTypeEnum
     * @return
     */
    ServiceResponse<Void> resetAuthResource(@NotNull Integer ownEnterpriseId, @NotNull Integer unionEnterpriseId, @NotNull Integer resourceType);

    /**
     * 解除联合
     * @param ownEnterpriseId
     * @param unionEnterpriseId
     * @param relieveReason 解除理由
     * @return
     */
    ServiceResponse<Void> relieveUnion(Integer ownEnterpriseId, Integer unionEnterpriseId, String relieveReason);

    /**
     * 联合商户列表
     * @param ownEnterpriseId 自有商户ID
     * @param enterpriseName 联合商户名称
     * @param authStatus UnionEnterpriseAuthDetailStatusEnum
     * @return
     */
    ServiceResponse<List<UnionEnterpriseAuthListDTO>> listUnionEnterpriseAuth(@NotNull Integer ownEnterpriseId, String enterpriseName, Integer authStatus);
}
